<?php

/*
 * PHPOF2/DBRow/Audited.php - PHPOF2 Audited database row
 *
 * (c)2001-2008 Tim Jackson (tim@timj.co.uk)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of version 3 of the GNU Lesser General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package PHPOF2
 */

require_once 'PHPOF2/DBRow.php';
require_once 'PHPOF2/Audit/Interface.php';

class PHPOF2_DBRow_Audited extends PHPOF2_DBRow
{
	public    $default_audit_owner = null; 
	protected $audit_owner = null;
	
	protected $audit_note = null;
	
	/**
	 * @var array List of fields to NOT audit (array of field names)
	 */
	protected $audit_ignore_fields = array();
	
	
	/**
	 * @var bool Temporarily disable auditing for this row
	 */
	protected $skip_audit = false;
	
	public function get($prikey_value = null)
	{
		$ret = parent::get($prikey_value);
		$this->_forceInitChangeRecording();
		return $ret;
	}
	
	public function insert()
	{
		parent::insert();
		$this->auditLog(PHPOF2_Audit_Interface::EVENT_TYPE_INSERT);
		$this->_forceInitChangeRecording();
	}
	
	public function update()
	{
		parent::update();
		$this->auditLog(PHPOF2_Audit_Interface::EVENT_TYPE_UPDATE);
		$this->_forceInitChangeRecording();
	}

	public function delete()
	{
		$this->auditLog(PHPOF2_Audit_Interface::EVENT_TYPE_DELETE);
		parent::delete();
		$this->_forceInitChangeRecording();
	}
	
	public function storeLOBData(&$data, $data_column)
	{
		$changes = array($data_column => '[Data changed]');
		$this->auditLog(PHPOF2_Audit_Interface::EVENT_TYPE_UPDATE, $changes);
		parent::storeLOBData($data, $data_column);
		$this->_forceInitChangeRecording();
	}
	
	public function fillFromResult(MDB2_Result &$result)
	{
		$ret = parent::fillFromResult($result);
		$this->_forceInitChangeRecording();
		return $ret;
	}
	
	public function setAuditNote($note)
	{
		$this->audit_note = $note;
	}
	
	public function setAuditOwner($owner)
	{
		$this->audit_owner = $owner;
	}
	
	protected function auditLog($type, $changes=null)
	{
		if (!$this->isAuditEnabled()) return;
		
		if ($changes === null) {
			$changes = $this->getChanges();
		}
		
		// Remove the ignore fields from the audit changes
		$changes = array_diff_key($changes, array_flip($this->audit_ignore_fields)); 

		$this->db->phpof2_auditor->recordEvent($type, $this->table->name, $this->serialisePriKey(), $this->_getAuditOwner(), $this->audit_note, $changes);
		$this->audit_note = null;
	}
	
	public function clear()
	{
		parent::clear();
		$this->_forceInitChangeRecording();
	}
	
	private function _getAuditOwner()
	{
		if ($this->audit_owner === null) {
			return $this->default_audit_owner;
		}
		return $this->audit_owner;
	}
	
	/*
	 * If the parent object doesn't have record changes set, we need to start 
	 * the recording
	 */ 
	protected function _forceInitChangeRecording()
	{
		if (!$this->record_changes) {
			$this->record_changes = true;
			$this->initChangeRecording();
			$this->record_changes = false;
		}
	}
	
	protected function clearAuditInformation()
	{
		$this->audit_note = null;
		$this->audit_owner = null;
	}
	
	/**
	 * Find out whether auditing is *currently* enabled for this row
	 * This takes into account both:
	 * - whether the audit driver is loaded
	 * - whether the "skip audit" property is set for this row
	 *
	 * @return bool Whether auditing is enabled
	 */
	function isAuditEnabled()
	{
		if ($this->skip_audit) return false;
		
		// In theory, $this->db->phpof2_auditor should always exist; however,
		// checking first and skipping if it doesn't means that auditing can
		// easily be disabled globally without having to change classes to
		// derive from PHPOF2_DBRow instead of PHPOF2_DBrow_Audited
		if (!isset($this->db->phpof2_auditor)) return false;
		
		return true;
	}

}
